﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class SoundManager: MonoBehaviour 
{
	#region Singleton
	static private SoundManager _instance;
	static public SoundManager GetInstance()
	{
		if(_instance == null)
		{
			_instance = new GameObject("SoundManager", typeof(SoundManager)).GetComponent<SoundManager>();
			_instance.LoadSounds();

			GameObject.DontDestroyOnLoad(_instance);
		}	
		return _instance; 
	}
	#endregion
	//Audio Sources
	public AudioSource				bgmAudioSource;
	public List<AudioSource>		sfxAudioSources;
	
	
	//Audio Clips
	public AudioClip				bgmClip;
	public List<AudioClip>			sfxClips;
	
	//Control
	public int			sfxSources = 3;
	public int			sfxCounter = 0;
	
	public void PlayBackGroundMusic()
	{
		bgmAudioSource.clip = bgmClip;
		bgmAudioSource.volume = 0.3f;
		bgmAudioSource.Play ();
	}
	public void StopBGM()
	{
		bgmAudioSource.Stop ();
	}

	public void PlaySFX(int p_index)
	{
		sfxAudioSources [sfxCounter].clip = sfxClips [p_index];
		sfxAudioSources [sfxCounter].Play ();
		sfxAudioSources [sfxCounter].volume = 0.5f;
		sfxCounter ++;
		if (sfxCounter == sfxAudioSources.Count)
			sfxCounter = 0;
	}
	
	void LoadSounds()
	{
		sfxAudioSources = new List<AudioSource> ();
		
		bgmAudioSource = new GameObject("BGMAudioSource", typeof(AudioSource)).GetComponent<AudioSource>();
		bgmAudioSource.loop = true;
		bgmAudioSource.transform.parent = _instance.transform;
		
		for (int i = 0; i < sfxSources; i ++) 
		{
			sfxAudioSources.Add(new GameObject ("SFXAudioSource" + i.ToString(), typeof(AudioSource)).GetComponent<AudioSource> ());
			sfxAudioSources[sfxAudioSources.Count -1].gameObject.transform.parent = _instance.transform;
		}

		bgmClip = Resources.Load<AudioClip> ("Audios/BGM/BGM_00");
		sfxClips = new List<AudioClip> ();
		for (int i = 0; i < 13; i++)
			sfxClips.Add(Resources.Load<AudioClip>("Audios/SFX/SFX_" + i.ToString()));
		
		
	}
}
