﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class CoreManager : MonoBehaviour 
{
	public GameObject			coreSpritesContainer;
	public AnimationManager		animationManager;
	public PlayerManager		playerManager;
	public EnemyManager			enemyManager;
	public CustomCollider2D		coreCollider;

	public List<Level>				levels;
	public List<SpriteRenderer> 	coreSprites;
	public SpriteRenderer			fullImage;

	public static int				level = 0;
	public static Level				levelInfo;
	public int						imageSet = 0;
	public int						stage = 0;
	public ColorReference.Colors	colorToTrigger;
	// Use this for initialization
	void Start () 
	{
		levelInfo = levels [level];
		SoundManager.GetInstance ().PlayBackGroundMusic ();
		fullImage.color = new Color (1f, 1f, 1f, 0f);
		coreSprites = new List<SpriteRenderer> ();
		SetUpCoreImages ();
		colorToTrigger = (ColorReference.Colors)Random.Range (0, 3);
		enemyManager.SetChances ();
		coreCollider.onTriggerEnter += delegate(CustomCollider2D arg1, Collider2D arg2) 
		{
			if (arg2.name.StartsWith("Enemy"))
				EnemyHitCore(arg2.GetComponent<Enemy>());
		};
		StartCoroutine(ChangeSpriteColor(coreSprites[0], coreSprites[0].color, GlobalInfo.GetInstance().colorReference.colorList[(int)colorToTrigger], 0.3f));
	}
	private void SetUpCoreImages()
	{
		stage = 0;
		foreach (SpriteRenderer __sr in coreSprites)
			Destroy (__sr.gameObject);
		coreSprites.Clear ();
		for(int i = 0; i < levels [level].coreImages [imageSet].imageSprites.Count; i++)
		{
			GameObject __go = new GameObject("Sprite" + i.ToString(), typeof(SpriteRenderer));
			__go.transform.SetParent(coreSpritesContainer.transform);
			__go.transform.localScale = Vector3.one;
			coreSprites.Add(__go.GetComponent<SpriteRenderer>());
			__go.GetComponent<SpriteRenderer>().sprite = levels [level].coreImages [imageSet].imageSprites[i];
			__go.GetComponent<SpriteRenderer>().color = new Color(0f,0f,0f,0f);
		}
		StartCoroutine(ChangeSpriteColor(coreSprites[0], coreSprites[0].color, GlobalInfo.GetInstance().colorReference.colorList[(int)colorToTrigger], 0.3f));
	}
	void Update ()
	{
		if (Input.GetKeyDown (KeyCode.Escape))
			Application.Quit ();
	}
	private void EnemyHitCore(Enemy p_enemy)
	{
		if (p_enemy.colors [p_enemy.colors.Count - 1] == colorToTrigger)
		{

			StartCoroutine(ChangeSpriteColor(coreSprites[stage], coreSprites[stage].color, Color.white, 0.3f));
			stage ++;

			if (stage < coreSprites.Count)
			{
				colorToTrigger = (ColorReference.Colors)Random.Range (0, 3);
				StartCoroutine(ChangeSpriteColor(coreSprites[stage - 1], Color.white, new Color(1f,1f,1f,0.2f), 1.5f));
				StartCoroutine(ChangeSpriteColor(coreSprites[stage], coreSprites[stage].color, GlobalInfo.GetInstance().colorReference.colorList[(int)colorToTrigger], 0.3f));
			}
			else 
			{
				CoreCompleted();

			}
		}
		else 
		{
			if (stage != 0)
			{
				colorToTrigger = (ColorReference.Colors)Random.Range (0, 3);
				StartCoroutine(ChangeSpriteColor(coreSprites[stage], coreSprites[stage].color, new Color(1f,1f,1f,0.0f), 0.1f));
				StartCoroutine(ChangeSpriteColor(coreSprites[stage - 1], coreSprites[stage - 1].color, GlobalInfo.GetInstance().colorReference.colorList[(int)colorToTrigger], 0.3f));
				stage--;
			}

		}
		SoundManager.GetInstance().PlaySFX(stage);
		p_enemy.EnemyHitCore ();
	}
	private void CoreCompleted()
	{
		imageSet ++;
		if (imageSet >= levels [level].coreImages.Count)
		{
			Time.timeScale = 1f;
			for (int i = 0; i < stage -1; i ++ )
				StartCoroutine(ChangeSpriteColor(coreSprites[i], coreSprites[i].color, Color.white, 1.5f));
			enemyManager.StopAndFadeEnemies();
			if (level != 0)
			{
				for (int i = 0; i < stage; i ++ )
					StartCoroutine(ChangeSpriteColor(coreSprites[i], Color.white, new Color(1f,1f,1f,0f), 4.5f));
				FullImageSetup();
			}

			playerManager.EnablePlayers(false);
			StartCoroutine(FinishLevel());

			coreCollider.enabled = false;
		}
		else
		{
			for (int i = 0; i < stage -1; i ++ )
				StartCoroutine(ChangeSpriteColor(coreSprites[i], coreSprites[i].color, Color.white, 1.5f));
			for (int i = 0; i < stage; i ++ )
				StartCoroutine(ChangeSpriteColor(coreSprites[i], Color.white, new Color(1f,1f,1f,0f), 4.5f));
			enemyManager.StopAndFadeEnemies();

			StartCoroutine(ChangeCore());
		}
	}
	private void FullImageSetup()
	{
		fullImage.sprite = levelInfo.fullImageSprite;
		fullImage.transform.SetParent(coreSpritesContainer.transform);
		fullImage.transform.localScale = levelInfo.fullImageScale;
		fullImage.transform.localPosition = levelInfo.fullImagePosition;
		StartCoroutine(ChangeSpriteColor(fullImage, fullImage.color, Color.white , 6.5f));
	}
	IEnumerator FinishLevel()
	{
		yield return new WaitForSeconds (6f);
		animationManager.LevelFinished();
		yield break;
	}
	IEnumerator ChangeCore()
	{
		yield return new WaitForSeconds (8f);
		SetUpCoreImages();
		enemyManager.SetActive (true);
		yield break;
	}
	IEnumerator ChangeSpriteColor(SpriteRenderer p_sprite, Color p_from, Color p_to, float p_delay)
	{
		yield return new WaitForSeconds (p_delay);
		float __t = 0f;
		
		while (__t < 1f)
		{
			__t += Time.deltaTime * 0.8f;
			p_sprite.color = Color.Lerp(p_from, p_to, __t);
			yield return null;
		}
		p_sprite.color = p_to;
		yield break;
	}

}
