﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using System;

public class Enemy : MonoBehaviour 
{
	public event Action<Enemy> 		OnEnemyDestroyed;
	public enum EnemyType
	{
		SIMPLE,
		MIXED,
		LAYER
	}
	public EnemyType type;
	public int 							layers = 1;
	public List<ColorReference.Colors> 	colors;
	public List<SpriteRenderer>			spriteRenderers;
	public bool							moving = true;
	//Last Hit for mixed and mixed-layers
	public ColorReference.Colors		lastHitColor = ColorReference.Colors.PURPLE;
	public int 							lastHisPlayerIndex =0;
	public float						lastHitTimer = 0f;
	private float						lastHit = 2f;

	private float 	_angle;
	private float 	_radOpos;
	private float	_speed = 0.5f;
	// Use this for initialization
	void Start () 
	{
		if (type == EnemyType.SIMPLE)
			colors.Add((ColorReference.Colors)UnityEngine.Random.Range (0, 3));
		else if (type == EnemyType.MIXED)
			colors.Add((ColorReference.Colors)UnityEngine.Random.Range (3, 6));
		else
		{
			colors.Add((ColorReference.Colors)UnityEngine.Random.Range (0, 6));
			for(int i = 1; i < layers; i ++)
				colors.Add((ColorReference.Colors)UnityEngine.Random.Range (0, 3));
		}
		foreach (Transform __trans in transform)
		{
			if (__trans.GetSiblingIndex () < layers) 
			{
				__trans.gameObject.SetActive (true);
				__trans.localScale = Vector3.one * (1f + (0.25f * __trans.GetSiblingIndex ()));
				spriteRenderers.Add(__trans.GetComponent<SpriteRenderer>());
			}
		}
		UpdateColors ();
	}
	void Update () 
	{
		if (!moving)
			return;
		transform.localPosition += new Vector3 (Mathf.Cos  (_radOpos), Mathf.Sin (_radOpos), 0f) * _speed * Time.deltaTime;
		if (lastHisPlayerIndex != 0)
		{
			lastHitTimer -= Time.deltaTime;
			if (lastHitTimer <= 0f)
				ResetLastHit();
		}
	}

	void OnTriggerEnter2D(Collider2D p_collision)
	{
		if (p_collision.name == "Shoot") 
			ShootHit(p_collision.GetComponent<Shoot>());
	}
	public void SetSpeed(float p_speed)
	{
		_speed = p_speed;
	}
	public void EnemyHitCore()
	{
		if (OnEnemyDestroyed != null)
			OnEnemyDestroyed (this);
	}
	private void ShootHit(Shoot p_shoot)
	{
		if (type == EnemyType.SIMPLE && p_shoot.color == colors[0])
		{
			if (OnEnemyDestroyed != null)
				OnEnemyDestroyed(this);
		}
		else if (type == EnemyType.MIXED)
		{
			if (CheckMixedColorMatch(p_shoot.color) && p_shoot.playerIndex != lastHisPlayerIndex)
			{
				if (lastHisPlayerIndex != 0 && p_shoot.color != lastHitColor && OnEnemyDestroyed != null)
					OnEnemyDestroyed(this);
				else
					SetLastHit(p_shoot.color, p_shoot.playerIndex);
			}
		}
		else if (type == EnemyType.LAYER)
		{
			//Simple Layer
			if ((int)colors[colors.Count -1] < 3 && p_shoot.color == colors[colors.Count -1])
			{
				if (colors.Count == 1 && OnEnemyDestroyed != null)
					OnEnemyDestroyed(this);
				else
					RemoveLayer();
			}
			//Mixed Layer
			else
			{
				if (CheckMixedColorMatch(p_shoot.color) && p_shoot.playerIndex != lastHisPlayerIndex)
				{
					if (lastHisPlayerIndex != 0 && p_shoot.color != lastHitColor && OnEnemyDestroyed != null)
					{
						if (colors.Count == 1)
							OnEnemyDestroyed(this);
						else
							RemoveLayer();

					}
					else
						SetLastHit(p_shoot.color, p_shoot.playerIndex);
				}
			}

		}
		p_shoot.ShootHitEnemy ();

	}
	private void RemoveLayer()
	{
		colors.RemoveAt (colors.Count - 1);
		spriteRenderers [spriteRenderers.Count - 1].gameObject.SetActive (false);
		spriteRenderers.RemoveAt (spriteRenderers.Count - 1);
		ResetLastHit ();
	}
	private void SetLastHit (ColorReference.Colors p_color, int p_playerIndex)
	{
		lastHitColor = p_color;
		lastHisPlayerIndex = p_playerIndex;
		lastHitTimer = lastHit;
	}
	private void ResetLastHit ()
	{
		lastHitColor = ColorReference.Colors.PURPLE;
		lastHisPlayerIndex = 0;
	}
	private bool CheckMixedColorMatch(ColorReference.Colors p_shootColor)
	{
		if (colors [colors.Count - 1] == ColorReference.Colors.GREEN && (p_shootColor == ColorReference.Colors.BLUE 
			|| p_shootColor == ColorReference.Colors.YELLOW))
			return true;
		else if (colors [colors.Count - 1] == ColorReference.Colors.PURPLE && (p_shootColor == ColorReference.Colors.BLUE 
		                                                                 || p_shootColor == ColorReference.Colors.RED))
			return true;
		else if (colors [colors.Count - 1] == ColorReference.Colors.ORANGE && (p_shootColor == ColorReference.Colors.RED 
		                                                                 || p_shootColor == ColorReference.Colors.YELLOW))
			return true;
		return false;
	}
	public void SetAngle(float p_angle)
	{
		_angle = p_angle;
		_radOpos = (180f + _angle) * Mathf.Deg2Rad;
		foreach (Transform __trans in transform)
			__trans.localRotation = Quaternion.Euler(0f,0f, _angle + 270f);
	}

	private void UpdateColors()
	{
		ColorReference __cr = GlobalInfo.GetInstance ().colorReference;
		for(int i = 0; i < layers; i++)
			spriteRenderers[i].color = __cr.colorList[(int)colors[i]];
	}
	public void FadeEnemy()
	{
		GetComponent<Collider2D> ().enabled = false;
		moving = false;
		foreach (SpriteRenderer __sr in spriteRenderers)
			StartCoroutine (FadeSprite (__sr,__sr.color));
		StartCoroutine (RemoveEnemy (3f));
	}
	IEnumerator FadeSprite(SpriteRenderer p_sr, Color p_from)
	{
		yield return new WaitForSeconds (0.3f);
		float __t = 0f;
		Color __to = new Color (p_from.r, p_from.g, p_from.b, 0f);
		while (__t < 1f)
		{
			__t += Time.deltaTime * 0.8f;
			p_sr.color = Color.Lerp(p_from, __to, __t);
			yield return null;
		}
		yield break;
	}
	IEnumerator RemoveEnemy(float p_delay)
	{
		yield return new WaitForSeconds (p_delay);
		if (OnEnemyDestroyed != null)
			OnEnemyDestroyed (this);
		yield break;
	}

}
