﻿using UnityEngine;
using System.Collections;

public class Player : MonoBehaviour 
{
	[SerializeField]
	private float angle;
	private float speed = 120f;

	private float shootCooldown = 0.41f;
	private float shootCooldownTimer = 0f;

	private float colorChange = 0.41f;
	private float colorChangeTimer = 0f;

	public ColorReference.Colors	color;
	public SpriteRenderer			spriteRenderer;
	// Use this for initialization
	void Start () 
	{
		shootCooldownTimer = 0.25f;
		colorChangeTimer = 0.25f;
		if (spriteRenderer == null)
			spriteRenderer = GetComponent<SpriteRenderer> ();
	}
	
	// Update is called once per frame
	void Update ()
	{
		shootCooldownTimer -= Time.deltaTime;
		colorChangeTimer -= Time.deltaTime;
	}
	public void ChangeColor(bool p_right)
	{
		colorChangeTimer = colorChange;
		if (p_right)
		{
			if ((int)color < 2)
				color ++;
			else
				color = ColorReference.Colors.RED;
		}
		else
		{
			if ((int)color > 0)
				color --;
			else
				color = ColorReference.Colors.YELLOW;
		}
		UpdateSpriteColor ();
	}
	public void SetColor(ColorReference.Colors p_color)
	{
		color = p_color;
		UpdateSpriteColor ();
	}
	public void UpdateSpriteColor()
	{
		if (spriteRenderer == null)
			spriteRenderer = GetComponent<SpriteRenderer> ();
		spriteRenderer.color = GlobalInfo.GetInstance ().colorReference.colorList [(int)color];
	}
	public bool GetChangeColorCooldown()
	{
		if (colorChangeTimer <= 0f)
			return true;
		return false;
	}
	public void Shoot()
	{
		if (shootCooldownTimer <= 0f)
			shootCooldownTimer = shootCooldown;
	}
	public bool GetShootCooldown()
	{
		if (shootCooldownTimer <= 0f)
			return true;
		return false;
	}
	public void MovePlayer(bool p_isRight)
	{
		if (p_isRight) 
			angle -= speed * Time.deltaTime;
		else
			angle += speed * Time.deltaTime;
		UpdatePosition ();
	}

	public void UpdatePosition()
	{
		float __rad = Mathf.Deg2Rad * angle;
		transform.localPosition = new Vector3 ((Mathf.Cos  (__rad)) * 1.23f, (Mathf.Sin (__rad)) * 1.23f, 0f);
		transform.localRotation = Quaternion.Euler (new Vector3 (0f, 0f, angle));
	}
	public void SetAngle(float p_angle)
	{
		angle = p_angle;
		UpdatePosition ();
	}
	public float GetAngle()
	{
		return angle;
	}
}
